#ifndef _COMPLEXLAYERNETWORK_H
#define _COMPLEXLAYERNETWORK_H

#include "MultiLayerNetwork.h"

namespace annie
{
/** Abstraction of an arbitrarily connected multi-layer perceptron network.
  * Essentially steals most of its implementation from MultiLayerNetwork
  *
  * All neurons in the layers are allowed to have a bias.
  * \todo The copy constructor
  */
class ComplexLayerNetwork : public MultiLayerNetwork
{
protected:
	//MultiLayerNetwork::int _nLayers;
	//MultiLayerNetwork::std::vector<Layer *> _layers;
	//MultiLayerNetwork::InputLayer *_inputLayer();
	//MultiLayerNetwork::Layer *_outputLayer();
public:
	/** Create a Complex-layer network.
	  * @param inputs The number of inputs taken in by the network
	  */
	ComplexLayerNetwork(int inputs) : MultiLayerNetwork(inputs) {}

	/// Copy constructor, NOT YET IMPLEMENTED
	ComplexLayerNetwork(ComplexLayerNetwork &srcNet);

	/** Loads a complex-layer network from the given filename.
	  * @param filename The name of the file from which to load the network
	  * @throws Exception On any error
	  */
	ComplexLayerNetwork(const char *filename);

	virtual ~ComplexLayerNetwork() {}

	/** Returns true if a connection between the two specified neurons is legal.
	  * @param srcLayer The index of the layer in which the source neuron lies
	  * @param srcNrn The index of the source neuron in the source layer
	  * @param destLayer The index of the layer in which the destination neuron lies
	  * @param destNrn The index of the destination neuron (will be in the layer with index srcLayer+1)
	  * 
	  */
	virtual bool checkConnection(int srcLayer, int srcNrn, int dstLayer, int destNrn);

	/// Returns "ComplexLayerNetwork"
	virtual const char *getClassName();
};
}; //namespace annie



#endif //_COMPLEXLAYERNETWORK_H
