#ifndef OURWORLD_H
#define OURWORLD_H

#include <iostream>
#include <vector>
#include <strstream>
#include "models/puck.hh"
#include "models/positiondevice.hh"
#include "models/gripperdevice.hh"
#include "models/laserdevice.hh"
#include <annie.h>
#include <ga.h>
#include <time.h>
#include <stdlib.h>

using namespace annie;
using namespace std;

int gTrialLength;


class OurWorld : public CStageServer
{

// convert meters to Player mm in NBO in various sizes
#define mMM_32(A)  (htonl((int32_t)(A * 1000.0)))
#define mMM_16(A)  (htons((int16_t)(A * 1000.0)))
#define mMM_U16(A) (htons((uint16_t)(A * 1000.0)))

// convert mm of various sizes in NBO to local meters
#define mM_32(A)  ((int)ntohl((int32_t)A) / 1000.0)
#define mM_16(A)  ((int)ntohs((int16_t)A) / 1000.0)
#define mM_U16(A) ((unsigned short)ntohs((uint16_t)A) / 1000.0)

// convert local radians to Player degrees in various sizes
#define mDeg_32(A) (htonl((int32_t)(RTOD(A))))
#define mDeg_16(A) (htons((int16_t)(RTOD(A))))
#define mDeg_U16(A) (htons((uint16_t)(RTOD(A))))

// convert Player degrees in various sizes to local radians
#define mRad_32(A) (DTOR((int)ntohl((int32_t)A)))
#define mRad_16(A) (DTOR((short)ntohs((int16_t)A)))
#define mRad_U16(A) (DTOR((unsigned short)ntohs((uint16_t)A)))

  vector<CPuck *> red;
  vector<CPuck *> blue;
  CPositionDevice *bot;
  CGripperDevice  *grip;
  CLaserDevice    *laser;
  int UsedPucks;

  StageColor b;
  StageColor r;

public:

  OurWorld( int argc, char** argv, Library* lib ) :  
    CStageServer( argc, argv, lib ), bot(NULL), grip(NULL), laser(NULL), UsedPucks(10)
  {

  }

  void DUMP() {
    cout << (bool)(grip->BreakBeam(0) || grip->BreakBeam(1)) << endl;
    sleep(1);
    Update();
  }

  double RunTrial(MultiLayerNetwork* net)
  {
    double rval = 0;
    int step = 0;
    int j;
    bool picked_up = false;
    VECTOR in;
    VECTOR out;

    in.resize(7);
    out.resize(4);
    
    ResetWorld();
    //    cout << "world setup" << endl;

    for(int i = 0; i < gTrialLength && !quit; i++) {
      //cout << "time step= " << i << " tlen= " << gTrialLength << endl;
      player_laser_data_t laser_data;
      laser->GenerateScanData(&laser_data);
      for(j = 0; j < 6; j++)
	in[j] = ntohs(laser_data.ranges[45*(j+2)])/800.0;
      
      in[j] = (bool)(grip->BreakBeam(0) || grip->BreakBeam(1));
      
      out = net->getOutput(in);

      //cout << "got output" << endl;

     if(out[2] > .8 && out[3] > .8){
     }
     else if(out[2] > .8) {
       double x=2.5,y=2.5,a=0;
       if(grip->m_puck_count) grip->m_pucks[grip->m_puck_count-1]->GetPose(x,y,a);
       if(x < .05 || 4.95 < x ||
	  y < .05 || 4.95 < y)
	 return 1;
       grip->open();
      }
      else if(out[3] > .8) {
	if(in[j])
	  picked_up=true;
	grip->close();
      }

     //     cout << "dealt with gripper" << endl;

      //should range about from -100,to 100
      double mL=(out[0]-.5)*200, mR=(out[1]-.5)*200;
      
      //Behold!  The ternary expression of DOOM!!!!! Muwhahahahaha...
      double newspeed = ((mL < 0 && mR < 0) || (mL > 0 && mR > 0))?((mL < 0)?max(mL,mR):min(mL,mR)):(mL + mR);
   
      //We did carefully calculate a divisor for this equation ...
      //But it produced lousy results, so use 3.4 instead- which
      //produced results that look right.
      double newturnrate = (mR - mL)/3.4;

      bot->com_vr = newspeed/1000.0;
      bot->com_vth = DTOR(newturnrate);

      //cout << "set speeds" << endl;

      CStageServer::Update();

      //cout << "updated" << endl;
    }

    return picked_up?.2:0;
  }

  void RoboGoHome() {
    bot->com_vr = bot->com_vth = 0;
    bot->SetPose(2.5,2.5,0);
    grip->open();
  }

  void ResetWorld() {
    RoboGoHome();
    vector<double> px(UsedPucks,0);
    vector<double> py(UsedPucks,0);
    for(int i=0;i<UsedPucks;i++) {
    too_close:
      px[i]=drand48()*4.7+.1;
      py[i]=drand48()*4.7+.1;
      if( (px[i]-2.5)*(px[i]-2.5) + 
	  (py[i]-2.5)*(py[i]-2.5) < .3)
	goto too_close;
      for(int k=0;k<i;k++) {
	if( (px[i]-px[k])*(px[i]-px[k]) + 
	    (py[i]-py[k])*(py[i]-py[k]) < .1)
	  goto too_close;
      }
    }

    cout << "puck positions generated" << endl;
    for(int i=0;i<UsedPucks;i++) {
      red[i]->SetPose(px[i],py[i],0);
    }
    cout << "pucks set" << endl;

  }

  void InitEverything() {

    r = LookupColor("red");
    b = LookupColor("blue");

    CPuck *puck=NULL;
    for(int i=0; i < entity_count; i++) {
      puck = dynamic_cast<CPuck*> (entities[i]);
      if(!bot)
	bot = dynamic_cast<CPositionDevice*> (entities[i]);
      if(!grip)
	grip = dynamic_cast<CGripperDevice*> (entities[i]);
      if(!laser)
	laser = dynamic_cast<CLaserDevice*> (entities[i]);
      if(puck) {
	if(puck->color == r)
	  red.push_back(puck);
	if(puck->color == b)
	  blue.push_back(puck);		
      }
    }
   if(bot)
	cout << "Got a bot" << endl;
   cout << "Got " << red.size() << " red pucks." << endl;
   cout << "Got " << blue.size() << " red pucks." << endl;
   bot->Subscribe();
  }

  //  pointers to all entities are also stored in a flat array
  //  be found by number without having to search the tree
  //protected: CEntity** entities;
  //  the number of entities stored in the array
  //protected: int entity_count; 


  //SetPose(double px, double py, double pth);
  //SetGlobalPose(double px, double py, double pth);
  //CEntity *m_parent_entity;
  //StageColor color;
};

TwoLayerNetwork gNet(7,10 ,4);

OurWorld *gWorld;

struct TrashWorldFitness
{
  static const bool favor_less = false;
  double operator()(const vector<double> &gene) const {
    
    cout << "starting fitness evaluation" << endl;
    cout << gene;    
    double rval = 0;
    for(int i = 0; i < 7; i++) {
      gNet.setWeights(gene);
      rval += gWorld->RunTrial(&gNet);
      cout << "trial done, rval= " << rval << " ";
    }

    cout << "returning" << endl;
    return rval;
  }
};


typedef GA<  TrashWorldFitness,    //fitness function
	     NolfiChildren<Const<double>, Gaussian, UniformDist>,
	     vector<double> >
ff_GA_P;  

struct ff_GA : public ff_GA_P {

  ff_GA(int vec_len) : 
    ff_GA_P( TrashWorldFitness(), 
	     NolfiChildren<Const<double>, Gaussian, UniformDist>
	      ( vec_len,
		Const<double>(.01), //mutation rate
		Gaussian(5),        //mutation mag
		UniformDist(20,-10))
	    )
  {}
};

void doMyBidding(OurWorld *w)
{
  srand48(time(NULL));
  
  gTrialLength = 1000;
  gWorld = w;
  gNet.connectAll();
  w->InitEverything();

  //while(!quit) w->DUMP();
  
  ofstream top_out("/scratch/sven/robo-final/ga-pop-backups/TwoLayer.best");

  ofstream all_fit("/scratch/sven/robo-final/ga-pop-backups/TwoLayer.fit");

  try {
    ff_GA g(124);
    for(int i=0;i<1000 && !quit;i++) {
  
      if(!(i%5)) {
	ostrstream os;
	os << "/scratch/sven/robo-final/ga-pop-backups/TwoLayerLog." 
	   << i << ".log" << '\0'; 
	ofstream out(os.str());
	for(int p=0;p<g.population.size();p++) out << g.population[p] << endl;
      }

      g.update_fitness();
      top_out << g.population_with_fitness[0].f << endl;
      top_out << g.population_with_fitness[0].gene << endl;

      for(int p=0;p<g.population_with_fitness.size();p++) {
	all_fit << g.population_with_fitness[p].f << ", ";
      }
      all_fit << endl;

      g.breed();
    }

  }

  catch (Exception e) {
    cout << e.what() << endl;
  }
  
}


#endif
