#ifndef OURWORLD_H
#define OURWORLD_H

#include <iostream>
#include <vector>
#include <strstream>
#include <fstream>
#include "models/puck.hh"
#include "models/positiondevice.hh"
#include "models/gripperdevice.hh"
#include "models/laserdevice.hh"
#include <annie.h>
#include <ga.h>
#include <time.h>
#include <stdlib.h>

using namespace annie;
using namespace std;

int gTrialLength;


struct RoboOrders
{
  double mL;
  double mR;
  double pickup;
  double drop;

};

struct NolfiOrderer {
  MultiLayerNetwork *net;
  NolfiOrderer(MultiLayerNetwork *n) : net(n) { }
  RoboOrders operator()(VECTOR in) {
    VECTOR out = net->getOutput(in);
    ASSERT(out.size()==16);    
    RoboOrders rv;
    rv.mL = out[0] > out[1] ? out[2] : out[3];
    rv.mR = out[4] > out[5] ? out[6] : out[7];
    rv.pickup = out[8] > out[9] ? out[10] : out[11];
    rv.drop = out[12] > out[13] ? out[14] : out[15];
    return rv;
  }
};

struct TwoLayerOrderer
{
  MultiLayerNetwork *net;
  TwoLayerOrderer(MultiLayerNetwork *n) : net(n) { }

  RoboOrders operator()(VECTOR in) {
    VECTOR out = net->getOutput(in);
      ASSERT(out.size()==4);    
    RoboOrders rv;
    rv.mL = out[0];
    rv.mR = out[1];
    rv.pickup = out[2];
    rv.drop = out[3];
    return rv;
  }

};

class OurWorld : public CStageServer
{

// convert meters to Player mm in NBO in various sizes
#define mMM_32(A)  (htonl((int32_t)(A * 1000.0)))
#define mMM_16(A)  (htons((int16_t)(A * 1000.0)))
#define mMM_U16(A) (htons((uint16_t)(A * 1000.0)))

// convert mm of various sizes in NBO to local meters
#define mM_32(A)  ((int)ntohl((int32_t)A) / 1000.0)
#define mM_16(A)  ((int)ntohs((int16_t)A) / 1000.0)
#define mM_U16(A) ((unsigned short)ntohs((uint16_t)A) / 1000.0)

// convert local radians to Player degrees in various sizes
#define mDeg_32(A) (htonl((int32_t)(RTOD(A))))
#define mDeg_16(A) (htons((int16_t)(RTOD(A))))
#define mDeg_U16(A) (htons((uint16_t)(RTOD(A))))

// convert Player degrees in various sizes to local radians
#define mRad_32(A) (DTOR((int)ntohl((int32_t)A)))
#define mRad_16(A) (DTOR((short)ntohs((int16_t)A)))
#define mRad_U16(A) (DTOR((unsigned short)ntohs((uint16_t)A)))

#define mntohs(s) ((((s) & 0xff) << 8) | (((s) >> 8) & 0xff))

  vector<CPuck *> red;
  vector<CPuck *> blue;
  CPositionDevice *bot;
  CGripperDevice  *grip;
  CLaserDevice    *laser;
  int UsedPucks;

  StageColor b;
  StageColor r;

public:

  OurWorld( int argc, char** argv, Library* lib ) :  
    CStageServer( argc, argv, lib ), bot(NULL), grip(NULL), laser(NULL), UsedPucks(5)
  {

  }

  void DUMP() {
    cout << (bool)(grip->BreakBeam(0) || grip->BreakBeam(1)) << endl;
    sleep(1);
    Update();
  }

  double adjustLaserValue(double lval)
  {
    lval=lval>300 ? lval-300 : 0;
    lval /= 100;
    if(lval>=1) {
      lval = (3 - lval)/2.0;
    }
    return lval;
  }
  
  template<class ordersFromNet>
  double quickTrial(MultiLayerNetwork* net)
  {
    int j;
    bool picked_up = false;
    VECTOR in;
    ordersFromNet calculate(net);
    in.resize(7);
    CEntity *n = NULL;
    
    ResetWorld();

    for(int i = 0; i < gTrialLength && !quit; i++) {

      player_laser_data_t laser_data;
      laser->GenerateScanData(&laser_data);
      for(j = 0; j < 6; j++) {
	int temp = laser_data.ranges[45*(j+2)];
	in[j] = adjustLaserValue(mntohs(temp));
	//cout << in[j] << " ";
      }
      //  cout << endl;
 
      in[j] = (bool)(grip->BreakBeam(0) || grip->BreakBeam(1));
      

      RoboOrders ord = calculate(in);

      if(ord.pickup > .8 && ord.drop > .8){
      }
      else if(ord.drop > .8) {
	CEntity * p = NULL;
	double x=2.5,y=2.5,a=0;
	if(grip->m_puck_count) {
	  p = grip->m_pucks[grip->m_puck_count-1];
	}
	
	grip->open();
	//n = NULL;
	if(p) {
	  p->GetPose(x,y,a);
	  p->laser_return = LaserVisible;
	  if(x < .05 || 4.95 < x ||
	     y < .05 || 4.95 < y)
	    return 1;
	}

      }
      else if(ord.pickup > .8) {
	CEntity *p = NULL;
	if(in[j])
	  picked_up=true;
	grip->close();
	if(grip->m_puck_count > 0) {
	  p = grip->m_pucks[grip->m_puck_count-1];
	  p->laser_return = LaserTransparent;
	}
	if(p && !n){	 
	  double x,y,a;
	  int i = 0;
	  do {
	    n = red[i++];
	  } while(n == p);
	  bot->GetPose(x,y,a);
	  x += .7*cos(a);
	  y += .7*sin(a);
	  if(.05 < x && x < 4.95 &&
	     .05 < y && y < 4.95)
	    n->SetPose(x, y,0);
	}
      }

      //should range about from -100,to 100
      double mL=(ord.mL-.5)*200, mR=(ord.mR-.5)*200;
      
      //Behold!  The ternary expression of DOOM!!!!! Muwhahahahaha...
      double newspeed = ((mL < 0 && mR < 0) || (mL > 0 && mR > 0))?((mL < 0)?max(mL,mR):min(mL,mR)):(mL + mR);
   
      //We did carefully calculate a divisor for this equation ...
      //But it produced lousy results, so use 3.4 instead- which
      //produced results that look right.
      double newturnrate = (mR - mL)/3.4;

      bot->com_vr = newspeed/1000.0;
      bot->com_vth = DTOR(newturnrate);

      CStageServer::Update();
    }

    return picked_up?.1:0;
  }

  template<class ordersFromNet>
  double fullTrial(MultiLayerNetwork* net)
  {
    double rval = 0;
    int j;
    int got_count=0;

    VECTOR in;
    ordersFromNet calculate(net);

    in.resize(7);
    
    ResetWorld();

    for(int i = 0; i < gTrialLength*4 && !quit; i++) {

      player_laser_data_t laser_data;
      laser->GenerateScanData(&laser_data);
      for(j = 0; j < 6; j++) {
	int temp = laser_data.ranges[45*(j+2)];
	in[j] = adjustLaserValue(mntohs(temp));
      }

      
      in[j] = (bool)(grip->BreakBeam(0) || grip->BreakBeam(1));
      
      RoboOrders ord = calculate(in);

      if(ord.pickup > .8 && ord.drop > .8){
      }
      else if(ord.drop > .8) {
	CEntity * p = NULL;
	double x=2.5,y=2.5,a=0;
	if(grip->m_puck_count)
	  p = grip->m_pucks[grip->m_puck_count-1];

	grip->open();       
	if(p) {
	  p->GetPose(x,y,a);
	  p->laser_return = LaserVisible;
	  if(x < .05 || 4.95 < x ||
	     y < .05 || 4.95 < y)
	    {
	      rval++;
	      p->SetPose(7.0,8.0-.1*got_count,0);
	      cout << got_count++ << endl;
	      //we have gotten all the pucks
	      if(got_count>=UsedPucks) return rval;
	    }
	}
      }
      else if(ord.pickup > .8) {
	grip->close();
	if(grip->m_puck_count > 0) 
	  grip->m_pucks[grip->m_puck_count-1]->laser_return = LaserTransparent;
      }

      //should range about from -100,to 100
      double mL=(ord.mL-.5)*200, mR=(ord.mR-.5)*200;
      
      //Behold!  The ternary expression of DOOM!!!!! Muwhahahahaha...
      double newspeed = ((mL < 0 && mR < 0) || (mL > 0 && mR > 0))?((mL < 0)?max(mL,mR):min(mL,mR)):(mL + mR);
   
      //We did carefully calculate a divisor for this equation ...
      //But it produced lousy results, so use 3.4 instead- which
      //produced results that look right.
      double newturnrate = (mR - mL)/3.4;

      bot->com_vr = newspeed/1000.0;
      bot->com_vth = DTOR(newturnrate);

      CStageServer::Update();
    }
    
    return rval;
  }

  void RoboGoHome() {
    bot->com_vr = bot->com_vth = 0;
    bot->SetPose(2.5,2.5,0);
    grip->open();
  }

  void ResetWorld() {
    RoboGoHome();
    vector<double> px(UsedPucks,0);
    vector<double> py(UsedPucks,0);
    for(int i=0;i<UsedPucks;i++) {
    too_close:
      px[i]=drand48()*4.7+.1;
      py[i]=drand48()*4.7+.1;
      if( (px[i]-2.5)*(px[i]-2.5) + 
	  (py[i]-2.5)*(py[i]-2.5) < .3)
	goto too_close;
      for(int k=0;k<i;k++) {
	if( (px[i]-px[k])*(px[i]-px[k]) + 
	    (py[i]-py[k])*(py[i]-py[k]) < .1)
	  goto too_close;
      }
    }


    for(int i=0;i<UsedPucks;i++) {
      red[i]->SetPose(px[i],py[i],0);
    }


  }

  void InitEverything() {

    r = LookupColor("red");
    b = LookupColor("blue");

    CPuck *puck=NULL;
    for(int i=0; i < entity_count; i++) {
      puck = dynamic_cast<CPuck*> (entities[i]);
      if(!bot)
	bot = dynamic_cast<CPositionDevice*> (entities[i]);
      if(!grip)
	grip = dynamic_cast<CGripperDevice*> (entities[i]);
      if(!laser)
	laser = dynamic_cast<CLaserDevice*> (entities[i]);
      if(puck) {
	if(puck->color == r)
	  red.push_back(puck);
	if(puck->color == b)
	  blue.push_back(puck);		
      }
    }
   if(bot)
	cout << "Got a bot" << endl;
   cout << "Got " << red.size() << " red pucks." << endl;
   cout << "Got " << blue.size() << " red pucks." << endl;
   bot->Subscribe();
  }

  //  pointers to all entities are also stored in a flat array
  //  be found by number without having to search the tree
  //protected: CEntity** entities;
  //  the number of entities stored in the array
  //protected: int entity_count; 


  //SetPose(double px, double py, double pth);
  //SetGlobalPose(double px, double py, double pth);
  //CEntity *m_parent_entity;
  //StageColor color;
};


MultiLayerNetwork * gNet;
OurWorld *gWorld;

template <class Orderer>
struct TrashWorldFitness
{
  static const bool favor_less = false;
  double operator()(const vector<double> &gene) const {
    
    cout << "starting fitness evaluation" << endl;
    //cout << gene;    
    double rval = 0;
    gNet->setWeights(gene);

    for(int i = 0; i < 15; i++) {
      rval += gWorld->template quickTrial<Orderer>(gNet);
      cout << "rval" << i << " " << rval << endl;
    }
    if(rval > 10) {
      cout << "doin a full trial" << endl;
      rval += gWorld->template fullTrial<Orderer>(gNet);
    }
    cout << "trial done: rval= " << rval << endl;
    return rval;
  }
};


template <class Orderer>
struct AnnieGA : 
  public GA< TrashWorldFitness<Orderer>,    //fitness function
	     NolfiChildren<Const<double>, Gaussian, UniformDist>,
	     vector<double> > {
  AnnieGA(int vec_len) : 
    GA< TrashWorldFitness<Orderer>,    //fitness function
	     NolfiChildren<Const<double>, Gaussian, UniformDist>,
	     vector<double> >
( TrashWorldFitness<Orderer>(), 
	NolfiChildren<Const<double>, Gaussian, UniformDist>
	( vec_len,
	  Const<double>(.01), //mutation rate
	  Gaussian(5),        //mutation mag
	  UniformDist(20,-10))
	)
  {}
};


void demoTwoLayer(OurWorld *w) {
  ifstream in("/scratch/sven/robo-final/TwoLayer.demo");
  cout << "Running demo for /scratch/sven/robo-final/TwoLayer.demo" << endl;

  TwoLayerNetwork tnet(7,10,4);
  tnet.connectAll();  
  gNet=&tnet;

  vector<double> v(124,0);
  for(int k=0;k<124;k++) 
    in >> v[k];

  while(!quit) TrashWorldFitness<TwoLayerOrderer>()(v);
}

void demoElman(OurWorld *w) {
  ifstream in("/scratch/sven/robo-final/Elman.demo");
  cout << "Running demo for /scratch/sven/robo-final/Elman.demo" << endl;
  
  ElmanNetwork tnet(7,6,4);
  tnet.connectAll();  
  gNet=&tnet;
  
  vector<double> v(112,0);
  for(int k=0;k<112;k++) 
    in >> v[k];

  while(!quit) TrashWorldFitness<TwoLayerOrderer>()(v);
}

void demoNolfi(OurWorld *w) {
  ifstream in("/scratch/sven/robo-final/Nolfi.demo");
  cout << "Running demo for /scratch/sven/robo-final/Nolfi.demo" << endl;
  
  MultiLayerNetwork tnet(7);
  tnet.addLayer(16);
  tnet.connectLayer(0);
  gNet=&tnet;

  vector<double> v(128,0);
  for(int k=0;k<128;k++) 
    in >> v[k];

  while(!quit) TrashWorldFitness<NolfiOrderer>()(v);
}


void TwoLayerExperiment() {
  int rseed=time(NULL);
  srand48(rseed);

  TwoLayerNetwork tnet(7,10,4);
  tnet.connectAll();  
  gNet=&tnet;
  ofstream top_out("/scratch/sven/robo-final/ga-pop-backups/TwoLayer/best");
  ofstream all_fit("/scratch/sven/robo-final/ga-pop-backups/TwoLayer/fit");


  top_out << "rseed: " << rseed << endl;

  AnnieGA<TwoLayerOrderer> g(124);
  for(int i=0;i<500 && !quit;i++) {
    
    if(!(i%5)) {
      ostrstream os;
      char num[10];
      sprintf(num,"%03d",i);
      os << "/scratch/sven/robo-final/ga-pop-backups/TwoLayer/generation." 
	 << num << ".log" << '\0'; 
      ofstream out(os.str());
      for(int p=0;p<g.population.size();p++) out << g.population[p] << endl;
    }
    
    g.update_fitness();
    top_out << g.population_with_fitness[0].f << endl;
    top_out << g.population_with_fitness[0].gene << endl;
    
    for(int p=0;p<g.population_with_fitness.size();p++) {
      all_fit << g.population_with_fitness[p].f << ", ";
    }
    all_fit << endl;
    
    g.breed();
  }    
}

void NolfiExperiment() {
  int rseed=time(NULL);
  srand48(rseed);

  MultiLayerNetwork tnet(7);
  tnet.addLayer(16);
  tnet.connectLayer(0);

  gNet=&tnet;
  ofstream top_out("/scratch/sven/robo-final/ga-pop-backups/Nolfi/best");
  ofstream all_fit("/scratch/sven/robo-final/ga-pop-backups/Nolfi/fit");

  top_out << "rseed: " << rseed << endl;

  AnnieGA<NolfiOrderer> g(128);
  for(int i=0;i<500 && !quit;i++) {
    
    if(!(i%5)) {
      ostrstream os;
      char num[10];
      sprintf(num,"%03d",i);
      os << "/scratch/sven/robo-final/ga-pop-backups/Nolfi/generation." 
	 << num << ".log" << '\0'; 
      ofstream out(os.str());
      for(int p=0;p<g.population.size();p++) out << g.population[p] << endl;
    }
    
    g.update_fitness();
    top_out << g.population_with_fitness[0].f << endl;
    top_out << g.population_with_fitness[0].gene << endl;
    
    for(int p=0;p<g.population_with_fitness.size();p++) {
      all_fit << g.population_with_fitness[p].f << ", ";
    }
    all_fit << endl;
    
    g.breed();
  }
    
}

void ElmanExperiment() {
  int rseed=time(NULL);
  srand48(rseed);

  ElmanNetwork tnet(7,6,4);
  tnet.connectAll();  
  gNet=&tnet;
  ofstream top_out("/scratch/sven/robo-final/ga-pop-backups/Elman/best");
  ofstream all_fit("/scratch/sven/robo-final/ga-pop-backups/Elman/fit");

  top_out << "rseed: " << rseed << endl;

  AnnieGA<TwoLayerOrderer> g(112);
  for(int i=0;i<500 && !quit;i++) {
    
    if(!(i%5)) {
      ostrstream os;
      char num[10];
      sprintf(num,"%03d",i);
      os << "/scratch/sven/robo-final/ga-pop-backups/Elman/generation." 
	 << num << ".log" << '\0'; 
      ofstream out(os.str());
      for(int p=0;p<g.population.size();p++) out << g.population[p] << endl;
    }
    
    g.update_fitness();
    top_out << g.population_with_fitness[0].f << endl;
    top_out << g.population_with_fitness[0].gene << endl;
    
    for(int p=0;p<g.population_with_fitness.size();p++) {
      all_fit << g.population_with_fitness[p].f << ", ";
    }
    all_fit << endl;
    
    g.breed();
  }
    
}


void ElmanResume() {
  int rseed=time(NULL);
  srand48(rseed);

  ElmanNetwork tnet(7,6,4);
  tnet.connectAll();  
  gNet=&tnet;
  ofstream top_out("/scratch/sven/robo-final/ga-pop-backups/Elman/bestResume");
  ofstream all_fit("/scratch/sven/robo-final/ga-pop-backups/Elman/fitResume");

  top_out << "rseed: " << rseed << endl;

  AnnieGA<TwoLayerOrderer> g(112);
  int gen = 0;

  {
    ifstream in("/scratch/sven/robo-final/Elman.resume");
    cout << "Resuming with info from /scratch/sven/robo-final/Elman.resume" << endl;
   
    in >> gen;

    ostrstream backupfilename;
    char num[10];
    sprintf(num,"%03d", gen);
    backupfilename << "/scratch/sven/robo-final/ga-pop-backups/Elman/generation." 
		   << num << ".log" << '\0'; 
    
    ifstream backupfile(backupfilename.str());
    

    for(int p=0;p<g.population.size();p++)
      for(int k=0;k<112;k++) 
	in >> g.population[p][k];


  }

  for(int i=gen;i<500 && !quit;i++) {
    
    if(!(i%5)) {
      ostrstream os;
      char num[10];
      sprintf(num,"%03d",i);
      os << "/scratch/sven/robo-final/ga-pop-backups/Elman/generation." 
	 << num << ".log" << '\0'; 
      ofstream out(os.str());
      for(int p=0;p<g.population.size();p++) out << g.population[p] << endl;
    }
    
    g.update_fitness();
    top_out << g.population_with_fitness[0].f << endl;
    top_out << g.population_with_fitness[0].gene << endl;
    
    for(int p=0;p<g.population_with_fitness.size();p++) {
      all_fit << g.population_with_fitness[p].f << ", ";
    }
    all_fit << endl;
    
    g.breed();
  }
    
}

void doMyBidding(OurWorld *w)
{

  
  gTrialLength = 1000;
  gWorld = w;
  w->InitEverything();

 
  try{
  //while(!quit) w->DUMP();

  //demoElman(w);
  //demoNolfi(w);
  //demoTwoLayer(w);

  //ElmanExperiment();
  //NolfiExperiment();
  //TwoLayerExperiment();

  ElmanResume(); 

  }

  catch (Exception e) {
    cout << e.what() << endl;
  }
}


#endif
